@@ -18,10 +18,6 @@ typedef struct nvme_effects_log_node {
 
 #define POWER_OF_TWO(exponent) (1 << (exponent))
 
-#define STR_LEN 100
-
-#define obj_add_str json_object_add_value_string
-
 void d(unsigned char *buf, int len, int width, int group);
 void d_raw(unsigned char *buf, unsigned len);
 
@@ -62,7 +58,7 @@ struct print_ops {
 	void (*media_unit_stat_log)(struct nvme_media_unit_stat_log *mus);
 	void (*mi_cmd_support_effects_log)(struct nvme_mi_cmd_supported_effects_log *mi_cmd_log, const char *devname);
 	void (*ns_list)(struct nvme_ns_list *ns_list);
-	void (*ns_list_log)(struct nvme_ns_list *log, const char *devname);
+	void (*ns_list_log)(struct nvme_ns_list *log, const char *devname, bool alloc);
 	void (*nvm_id_ns)(struct nvme_nvm_id_ns *nvm_ns, unsigned int nsid, struct nvme_id_ns *ns, unsigned int lba_index, bool cap_only);
 	void (*persistent_event_log)(void *pevent_log_info, __u8 action, __u32 size, const char *devname);
 	void (*predictable_latency_event_agg_log)(struct nvme_aggregate_predictable_lat_event *pea_log, __u64 log_entries, __u32 size, const char *devname);
@@ -92,6 +88,15 @@ struct print_ops {
 	void (*d)(unsigned char *buf, int len, int width, int group);
 	void (*show_init)(void);
 	void (*show_finish)(void);
+	void (*mgmt_addr_list_log)(struct nvme_mgmt_addr_list_log *ma_log);
+	void (*rotational_media_info_log)(struct nvme_rotational_media_info_log *info);
+	void (*dispersed_ns_psub_log)(struct nvme_dispersed_ns_participating_nss_log *log);
+	void (*reachability_groups_log)(struct nvme_reachability_groups_log *log, __u64 len);
+	void (*reachability_associations_log)(struct nvme_reachability_associations_log *log,
+					      __u64 len);
+	void (*host_discovery_log)(struct nvme_host_discover_log *log);
+	void (*ave_discovery_log)(struct nvme_ave_discover_log *log);
+	void (*pull_model_ddc_req_log)(struct nvme_pull_model_ddc_req_log *log);
 
 	/* libnvme tree print functions */
 	void (*list_item)(nvme_ns_t n);
@@ -103,7 +108,7 @@ struct print_ops {
 	/* status and error messages */
 	void (*connect_msg)(nvme_ctrl_t c);
 	void (*show_message)(bool error, const char *msg, va_list ap);
-	void (*show_perror)(const char *msg);
+	void (*show_perror)(const char *msg, va_list ap);
 	void (*show_status)(int status);
 	void (*show_error_status)(int status, const char *msg, va_list ap);
 
@@ -176,8 +181,8 @@ void nvme_show_fw_log(struct nvme_firmware_slot *fw_log, const char *devname,
 	nvme_print_flags_t flags);
 void nvme_print_effects_log_pages(struct list_head *list,
 				  nvme_print_flags_t flags);
-void nvme_show_changed_ns_list_log(struct nvme_ns_list *log,
-	const char *devname, nvme_print_flags_t flags);
+void nvme_show_changed_ns_list_log(struct nvme_ns_list *log, const char *devname,
+				   nvme_print_flags_t flags, bool alloc);
 void nvme_show_endurance_log(struct nvme_endurance_group_log *endurance_log,
 	__u16 group_id, const char *devname, nvme_print_flags_t flags);
 void nvme_show_sanitize_log(struct nvme_sanitize_log_page *sanitize,
@@ -314,11 +319,14 @@ const char *nvme_plm_window_to_string(__u32 plm);
 const char *nvme_ns_wp_cfg_to_string(enum nvme_ns_write_protect_cfg state);
 const char *nvme_pel_rci_rcpit_to_string(enum nvme_pel_rci_rcpit rcpit);
 const char *nvme_pel_ehai_pit_to_string(enum nvme_pel_ehai_pit pit);
+const char *nvme_ssi_state_to_string(__u8 state);
+const char *nvme_time_scale_to_string(__u8 ts);
+const char *nvme_pls_mode_to_string(__u8 mode);
 
 void nvme_dev_full_path(nvme_ns_t n, char *path, size_t len);
 void nvme_generic_full_path(nvme_ns_t n, char *path, size_t len);
 void nvme_show_message(bool error, const char *msg, ...);
-void nvme_show_perror(const char *msg);
+void nvme_show_perror(const char *msg, ...);
 void nvme_show_error_status(int status, const char *msg, ...);
 void nvme_show_init(void);
 void nvme_show_finish(void);
@@ -328,12 +336,22 @@ bool nvme_registers_cmbloc_support(__u32 cmbsz);
 bool nvme_registers_pmrctl_ready(__u32 pmrctl);
 const char *nvme_degrees_string(long t);
 void print_array(char *name, __u8 *data, int size);
-void obj_add_uint_02x(struct json_object *o, const char *k, __u32 v);
 void json_print(struct json_object *r);
-void obj_add_uint_0x(struct json_object *o, const char *k, __u32 v);
-void obj_add_byte_array(struct json_object *o, const char *k, unsigned char *buf, int len);
-void obj_add_nprix64(struct json_object *o, const char *k, uint64_t v);
 struct json_object *obj_create_array_obj(struct json_object *o, const char *k);
-void obj_add_uint_0nx(struct json_object *o, const char *k, __u32 v, int width);
-void obj_add_0nprix64(struct json_object *o, const char *k, uint64_t v, int width);
+void nvme_show_mgmt_addr_list_log(struct nvme_mgmt_addr_list_log *ma_list,
+				  nvme_print_flags_t flags);
+void nvme_show_rotational_media_info_log(struct nvme_rotational_media_info_log *info,
+					 nvme_print_flags_t flags);
+void nvme_show_dispersed_ns_psub_log(struct nvme_dispersed_ns_participating_nss_log *log,
+				     nvme_print_flags_t flags);
+void nvme_show_reachability_groups_log(struct nvme_reachability_groups_log *log,
+				       __u64 len, nvme_print_flags_t flags);
+void nvme_show_reachability_associations_log(struct nvme_reachability_associations_log *log,
+					     __u64 len, nvme_print_flags_t flags);
+void nvme_show_host_discovery_log(struct nvme_host_discover_log *log, nvme_print_flags_t flags);
+void nvme_show_ave_discovery_log(struct nvme_ave_discover_log *log, nvme_print_flags_t flags);
+void nvme_show_pull_model_ddc_req_log(struct nvme_pull_model_ddc_req_log *log,
+				      nvme_print_flags_t flags);
+
+extern char *alloc_error;
 #endif /* NVME_PRINT_H */
