@@ -31,6 +31,7 @@
 #include "util/mem.h"
 #include "util/argconfig.h"
 #include "util/cleanup.h"
+#include "util/types.h"
 
 enum nvme_print_flags {
 	NORMAL		= 0,
@@ -147,7 +148,6 @@ const char *nvme_select_to_string(int sel);
 
 void d(unsigned char *buf, int len, int width, int group);
 void d_raw(unsigned char *buf, unsigned len);
-uint64_t int48_to_long(uint8_t *data);
 
 int get_reg_size(int offset);
 bool nvme_is_ctrl_reg(int offset);
